import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import org.apache.xerces.dom.DOMImplementationImpl;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;

public class Gradebook extends JFrame
implements ActionListener, ListSelectionListener,
DocumentListener, KeyListener, FocusListener {

	JMenuBar	jmb;
	static final int FILE_MENU=0;
	static final int TASK_MENU=1;
	static final int STUDENT_MENU=2;

	boolean		dirty;

	TaskManager	taskManager;
	int			currentTask;

	StudentManager studentManager;
	int			   currentStudent;

	JScrollPane	taskPane;
	JScrollPane	studentPane;

	JTextField	score;
	JTextArea	comments;
	JButton		updateButton;

    Document	gradebookDocument;
	Element		gradebookRoot;
	String		xmlFileName;


	public Gradebook()
	{
		JMenu		fileMenu = new JMenu("File");
		JMenu		taskMenu = new JMenu("Tasks");
		JMenu		studentMenu = new JMenu("Students");
		JPanel		scoreEntry;

		GridBagLayout	gbl;
		GridBagConstraints gbc;
		DefaultListModel	model;

		String fileItems[] = new String[] {
			"New", "Open/control O", "Save/control S",
			"Save As...", "--", "Quit/control Q"
		};

		String taskItems[] = new String[] {
			"Add Task...",
			"Edit Task...", "Delete Task..."
		};

		String studentItems[] = new String[] {
			"Add Student...", "Edit Student...",
			"Delete Student...", "--", "Import..."
		};

		taskManager = null;
		studentManager = null;
		currentTask = -1;
		currentStudent = -1;

		newFile();
		modifiedFile( false );

		setupMenu( fileMenu, fileItems );
		setupMenu( taskMenu, taskItems );
		setupMenu( studentMenu, studentItems );
		jmb = new JMenuBar();
		jmb.add(fileMenu);
		jmb.add(taskMenu);
		jmb.add(studentMenu);
		this.setJMenuBar( jmb );
		resetMenu();

		/* Set up the layout for the frame */
		gbl = new GridBagLayout();
		gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 100;
		gbc.weighty = 100;

		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets( 2, 2, 2, 2 );

		this.getContentPane().setLayout( gbl );
		model = new DefaultListModel();

		taskManager = new TaskManager( gradebookDocument );
		taskManager.getList().addActionListener( this );
		taskManager.getList().addListSelectionListener( this );
		taskPane = new JScrollPane( taskManager.getList() );

		addToGridBag( taskPane, gbl, gbc, 0, 0, 1, 4 );
		model = new DefaultListModel();

		studentManager = new StudentManager( gradebookDocument );
		studentManager.getList().addActionListener( this );
		studentManager.getList().addListSelectionListener( this );

		studentPane = new JScrollPane( studentManager.getList() );
		addToGridBag( studentPane, gbl, gbc, 1, 0, 2, 1 );

		JLabel label = new JLabel("Score:");
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0;
		addToGridBag( label, gbl, gbc, 1, 1, 1, 1);

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 100;
		score = new JTextField( 10 );
		score.getDocument().addDocumentListener(this);
		score.addKeyListener(this);
		score.addFocusListener(this);
		score.setEnabled(false);

		addToGridBag( score, gbl, gbc, 2, 1, 1, 1 );

		label = new JLabel("Comments:");
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0;
		addToGridBag( label, gbl, gbc, 1, 2, 1, 1);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 100;
		comments = new JTextArea( 5, 30 );
		comments.getDocument().addDocumentListener(this);
		comments.addFocusListener(this);
		comments.setLineWrap( true );
		comments.setWrapStyleWord( true );
		comments.setFont( new Font( "Monospaced", Font.PLAIN, 10 ) );
		comments.setEnabled( false );
		addToGridBag( new JScrollPane(comments), gbl, gbc, 2, 2,
			1, 1 );

		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0;
		updateButton = new JButton("Update");
		updateButton.setEnabled( false );
		updateButton.addActionListener( this );
		addToGridBag( updateButton, gbl, gbc, 1, 3, 2, 1 );

		this.setTitle("Gradebook - New File");

	}

	private void addToGridBag( Component c, GridBagLayout gbl,
		GridBagConstraints gbc,
		int x, int  y, int w, int h )
	{
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight= h;
		gbl.setConstraints( c, gbc );
		this.getContentPane().add( c );
	}

	private void setWeights( GridBagConstraints gbc, int xweight, int yweight )
	{
		gbc.weightx = xweight;
		gbc.weighty = yweight;
	}

	private void setupMenu( JMenu menu, String[] itemlist )
	{
		JMenuItem	jmi;
		int			i;
		int			pos;
		String		message, accelerator;

		for (i=0; i < itemlist.length; i++)
		{
			if (itemlist[i].equals("--"))
			{
				menu.insertSeparator(i+1);
			}
			else
			{
				message = itemlist[i];
				accelerator = "";
				pos = itemlist[i].indexOf( '/' );
				if (pos >= 0)
				{
					message = itemlist[i].substring( 0, pos );
					accelerator = itemlist[i].substring( pos+1 );
				}
				jmi = new JMenuItem(message);
				if (!accelerator.equals(""))
				{
					jmi.setAccelerator( KeyStroke.getKeyStroke(
						accelerator ) );
				}
				jmi.addActionListener( this );
				menu.add( jmi );
			}
		}
	}

	public void actionPerformed(ActionEvent event)
	{
		String	command = event.getActionCommand();

		if (command.equals("New"))
		{
			if (checkForSave("Save first?"))
			{
				newFile();
				resetMenu();
			}
		}
		else if (command.equals("Open"))
		{
			if (checkForSave("Save first?"))
			{
				openFile();
			}
		}
		else if (command.equals("Save"))
		{
			saveFile( xmlFileName );
		}
		else if (command.equals("Save As..."))
		{
			saveFile( "" );
		}
		else if (command.equals("Quit"))
		{
			if (checkForSave("Save before Quit?"))
			{
				System.exit(0);
			}
		}
		else if (command.equals("Add Student..."))
		{
			if (studentManager.editStudent( -1 ))
			{
				modifiedFile( true );
			}
		}
		else if (command.equals("Delete Student..."))
		{
			if (studentManager.deleteStudent())
			{
				modifiedFile( true );
			}
		}
		else if (command.equals("Edit Student..."))
		{
			if (studentManager.
				editStudent( currentStudent ))
			{
				modifiedFile( true );
			};
		}
		else if (command.equals("Import..."))
		{
			studentManager.importTextFile();
		}
		else if (command.equals("Update"))
		{
			performUpdate();
		}
		else if (command.equals("Add Task..."))
		{
			if (taskManager.editTask(-1))
			{
				modifiedFile( true );
			}
		}
		else if (command.equals("Edit Task..."))
		{
			if (taskManager.editTask( currentTask ))
			{
				modifiedFile( true );
			}
		}
		else if (command.equals("Delete Task..."))
		{
			if (taskManager.deleteTask())
			{
				modifiedFile( true );
			}
		}
		else if (command.startsWith("[")) // doubleclicked list item
		{
			Object obj = event.getSource();
			if (obj == studentManager.getList())
			{
				studentManager.editStudent( currentStudent );
			}
			else if (obj == taskManager.getList())
			{
				taskManager.editTask( currentTask );
			}
		}
	}

	public void performUpdate()
	{
		String oldScore, oldComments;
		if (currentStudent >= 0 && currentTask >= 0)
		{
			Element result = studentManager.getResult(
				currentStudent, currentTask
			);
			oldScore = result.getAttribute("score");
			oldComments = DOMUtil.findValue( result );
			if (!(oldScore.equals(score.getText()) &&
				oldComments.equals(comments.getText())))
			{
				studentManager.setResult( result,
					score.getText(),
					comments.getText());
				modifiedFile( true );
			}
			updateButton.setEnabled(false);
		}
	}

	public void valueChanged( ListSelectionEvent e )
	{
		int	task, stu;
		stu = studentManager.getList().getSelectedIndex();
		task = taskManager.getList().getSelectedIndex();

		if (updateButton.isEnabled())
		{
			performUpdate();
		}
		currentStudent = stu;
		currentTask = task;
		if (e.getSource() == taskManager.getList())
		{
			if (taskManager.getList().getSelectedIndex() >= 0)
			{
				jmb.getMenu( TASK_MENU ).getItem(1).setEnabled( true );
				jmb.getMenu( TASK_MENU ).getItem(2).setEnabled( true );
			}
		}
		else if (e.getSource() == studentManager.getList())
		{
			if (studentManager.getList().getSelectedIndex() >= 0)
			{
				jmb.getMenu( STUDENT_MENU ).getItem(1).setEnabled( true );
				jmb.getMenu( STUDENT_MENU ).getItem(2).setEnabled( true );
			}
		}
		if (stu >= 0 && task >= 0)
		{
			Element result = studentManager.getResult( stu, task );
			if (result != null)
			{
				String commentText = DOMUtil.findValue( result );
				commentText = DOMUtil.normalizeSpace( commentText );
				score.setText( result.getAttribute("score") );
				comments.setText( commentText.
					replace('|', '\n') );
				score.setEnabled(true);
				comments.setEnabled(true);
			}
			else
			{
				score.setText("");
				comments.setText("");
				score.setEnabled(false);
				comments.setEnabled(false);
			}
		}
		else
		{
			score.setText("");
			comments.setText("");
		}
		updateButton.setEnabled( false );
		jmb.getMenu( STUDENT_MENU ).getItem(1).setEnabled( stu >= 0 );
		jmb.getMenu( STUDENT_MENU ).getItem(2).setEnabled( stu >= 0 );
		jmb.getMenu( TASK_MENU ).getItem(1).setEnabled( task >= 0 );
		jmb.getMenu( TASK_MENU ).getItem(2).setEnabled( task >= 0 );
	}

	private void saveFile( String fileName )
	{
        FileWriter out = null;

		if (fileName.equals(""))
		{
			JFileChooser chooser = new JFileChooser();
			int	result;

			chooser.setCurrentDirectory(
				new File( System.getProperty("user.dir") ) );
			result = chooser.showSaveDialog( this );
			if (result != JFileChooser.APPROVE_OPTION ||
				chooser.getSelectedFile() == null)
			{
				return;
			}
			fileName = chooser.getSelectedFile().getAbsolutePath();
		}
        try{
            out =
            new FileWriter(fileName);
        }
        catch (Exception e)
        {
            System.out.println("Error creating output stream");
            System.out.println(e.getMessage());
            alert("Unable to create output file");
			return;
        }
        OutputFormat oFormat = new OutputFormat( "xml", "UTF-8", true );
        XMLSerializer serial = new XMLSerializer( out, oFormat );
        try
        {
            serial.serialize( gradebookDocument );
        }
        catch (java.io.IOException e)
        {
            System.out.println(e.getMessage());
        }
		xmlFileName = fileName;
		modifiedFile( false );
	}

	private void newFile()
	{
		Node	node;
		Element	element;
		DOMImplementation dImplement = new DOMImplementationImpl();

		gradebookDocument = dImplement.createDocument(null,
			"gradebook", null);
		gradebookRoot = gradebookDocument.getDocumentElement();
		element = gradebookDocument.createElement("task-list");
		gradebookRoot.appendChild( element );
		setDocuments( gradebookDocument );
		xmlFileName = "";

		modifiedFile( false );
	}

	private void resetMenu()
	{
		jmb.getMenu( TASK_MENU ).getItem(1).setEnabled( false );
		jmb.getMenu( TASK_MENU ).getItem(2).setEnabled( false );
		jmb.getMenu( STUDENT_MENU ).getItem(1).setEnabled( false );
		jmb.getMenu( STUDENT_MENU ).getItem(2).setEnabled( false );
	}

	private void openFile()
	{
		JFileChooser chooser = new JFileChooser();
		DOMParser parser = null;
		int	result;
		String fileName;

		chooser.setCurrentDirectory(
			new File( System.getProperty("user.dir") ) );
		result = chooser.showOpenDialog( this );
		if (result != JFileChooser.APPROVE_OPTION ||
			chooser.getSelectedFile() == null)
		{
			return;
		}

		fileName = chooser.getSelectedFile().getAbsolutePath();

        ParserErrorHandler errHandler = new ParserErrorHandler();
        gradebookDocument = null;
        try {
            parser = new
				org.apache.xerces.parsers.DOMParser();
            parser.setErrorHandler( errHandler );
            parser.parse( fileName );
 			if (!parser.getDocument().
				getDocumentElement().getTagName().equals("gradebook"))
			{
				alert("This does not appear to be a gradebook file.");
				return;
			}
        } catch ( Exception e ) {
            e.printStackTrace(System.err);
        }
		gradebookDocument = parser.getDocument();
		gradebookRoot = gradebookDocument.getDocumentElement();
		taskManager.setDocument( gradebookDocument );
		studentManager.setDocument( gradebookDocument );
		xmlFileName = fileName;
		this.setTitle( xmlFileName );
	}

	private void setDocuments( Document doc )
	{
		if (taskManager != null)
		{
			taskManager.setDocument( doc );
		}

		if (studentManager != null)
		{
			studentManager.setDocument( doc );
		}
	}

	public boolean checkForSave(String prompt)
	{
		if (dirty)
		{
			int result;
			result = JOptionPane.showConfirmDialog( this, prompt );
			if (result == JOptionPane.YES_OPTION)
			{
				saveFile( xmlFileName );
			}
			return (result != JOptionPane.CANCEL_OPTION);
		}
		else
		{
			return true;
		}
	}
	
	public void modifiedFile( boolean b )
	{
		String	s = xmlFileName;
		if (s.equals(""))
		{
			s = "Gradebook - New File";
		}
		dirty = b;
		if (dirty) { s += " (modified)"; }
		this.setTitle( s );
	}	

	public void alert( String message )
	{
		JOptionPane.showMessageDialog( this, message );
	}

	public void changedUpdate(DocumentEvent e)
	{
		updateButton.setEnabled(true);
	}

	public void insertUpdate(DocumentEvent e)
	{
		updateButton.setEnabled(true);
	}

	public void removeUpdate(DocumentEvent e)
	{
		updateButton.setEnabled(true);
	}

	public void keyPressed(KeyEvent e) { }
	public void keyTyped(KeyEvent e) { }
	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			performUpdate();
		}
	}

	public void focusLost(FocusEvent e)
	{
		performUpdate();
	}

	public void focusGained(FocusEvent e) { };

	public static void main(String args[]) {
		Gradebook g = new Gradebook();
		g.addWindowListener(new BasicWindowMonitor(g));
		g.setSize( 600, 400 );
		g.setVisible( true );
	}
}
