import java.awt.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class StudentManager
{
	Document		document;
	DoubleClickList	list;
	DefaultListModel model;

	/**
	 * Create a Student List Manager with a given document
	 */
	public StudentManager( Document doc )
	{
		document = doc;
		model = new DefaultListModel();
		list = new DoubleClickList( model );
	}

	public Document getDocument()
	{
		return document;
	}

	public void setDocument( Document doc )
	{
		document = doc;
		clearList();
		populateList();
	}

	public DoubleClickList getList()
	{
		return list;
	}

	public void clearList( )
	{
		if (model != null)
		{
			model.removeAllElements();
		}
	}


	public void populateList( )
	{
		NodeList	studentNames;
		Node		student;
		Node		temp;
		NodeList	studentInfo;
		String		str;

		int i;
		if (model == null || document == null)
		{
			System.out.println("no model");
			return;
		}

		studentNames = document.getElementsByTagName( "student" );
		for (i=0; i < studentNames.getLength(); i++)
		{
			str = "";
			student = studentNames.item( i );
			str = DOMUtil.findChildValue( student, "last" ) + ", " +
				DOMUtil.findChildValue( student, "first" ) +
				" [" + DOMUtil.findAttrValue( student, "id" ) + "]";
			model.addElement( str );
		}
	}

	public int addStudent( Element student )
	{
		NodeList	studentNames;
		Element		el;
		int			i;
		int			comparison;
		String		nodeFirst, nodeLast, nodeID;
		String		fname, lname, id;
		String		listStr;
	 
	 	fname = DOMUtil.findChildValue( student, "first" );
		lname = DOMUtil.findChildValue(	student, "last" );
		id = ((Element) student).getAttribute("id");

		listStr = lname + ", " + fname + " [" + id + "]";

		studentNames = document.getElementsByTagName( "student" );

		if (studentNames.getLength() != 0)
		{
			i = 0;
			while (i < studentNames.getLength())
			{
				nodeFirst = DOMUtil.findChildValue(
					studentNames.item(i), "first" );
				nodeLast = DOMUtil.findChildValue(
					studentNames.item(i), "last" );
				nodeID =
					((Element) studentNames.item(i)).getAttribute("id");
				comparison = lname.compareTo(nodeLast);
				if (comparison == 0)
				{
					comparison = fname.compareTo(nodeFirst);
					if (comparison == 0)
					{
						comparison = id.compareTo(nodeID);
					}
				}

				if (comparison > 0)
				{
					i++;
					continue;
				}
				if (comparison == 0)
				{
					JOptionPane.showMessageDialog(
						list.getTopLevelAncestor(),
						"Duplicate entry " + fname + " " + lname +
						" skipped." );
					return -1;
				}
				document.getDocumentElement().insertBefore(
					student, studentNames.item(i));
				model.insertElementAt( listStr, i );
				list.ensureIndexIsVisible(i);
				return i;
			}
			document.getDocumentElement().appendChild( student );
			model.addElement( listStr );
			list.ensureIndexIsVisible(i);
			return 0;
		}
		else
		{
			document.getDocumentElement().appendChild( student );
			model.addElement( listStr );
			return 0;
		}
	}

	public boolean deleteStudent()
	{
		int		n = list.getSelectedIndex();
		int		result;
		Element	student;

		if (n < 0)
		{
			return false;
		}
		student = (Element)
				document.getElementsByTagName("student").item(n);

		if (student == null) { return false; }
		result = JOptionPane.showConfirmDialog(
			list.getTopLevelAncestor(),
			"Really delete "
			+  DOMUtil.findChildValue( student, "first" ) +  " "
			+ DOMUtil.findChildValue( student, "last" ) + "["
			+ student.getAttribute("id") + "]?");
		if (result == JOptionPane.YES_OPTION)
		{
			student.getParentNode().removeChild( student );
			model.removeElementAt( n );
			list.clearSelection();
			return true;
		}
		return false;
	}

	public Element updateStudent( Element baseStudent,
		String lname,
		String fname,
		String id,
		String email,
		String info )
	{
		NodeList	taskNames;
		Element		el;
		DocumentFragment df;
		int			i;

		if (baseStudent != null)
		{
			baseStudent.setAttribute( "id", id );
			DOMUtil.setText( baseStudent, "last", lname );
			DOMUtil.setText( baseStudent, "first", fname );
			DOMUtil.setText( baseStudent, "email", email );
			DOMUtil.setText( baseStudent, "info", info );
		}
		else
		{
			baseStudent = document.createElement( "student" );
			baseStudent.setAttribute( "id", id );
			df = DOMUtil.createTextElement( document, "last", lname );
			baseStudent.appendChild( df );
			df = DOMUtil.createTextElement( document, "first", fname );
			baseStudent.appendChild( df );
			df = DOMUtil.createTextElement( document, "email", email );
			baseStudent.appendChild( df );
			df = DOMUtil.createTextElement( document, "info", info );
			baseStudent.appendChild( df );
			taskNames = document.getElementsByTagName( "task" );

			for (i=0; i < taskNames.getLength(); i++)
			{
				el = document.createElement( "result" );
				el.setAttribute( "ref",
				((Element) taskNames.item(i)).getAttribute("id") );
				baseStudent.appendChild( el );
			}
		}

		return baseStudent;
	}

	protected Element studentDialog( Element student )
	{
		String	lname="";
		String	fname="";
		String	email="";
		String	id="";
		String	info="";
		JOptionPane optPane;
		JTextField lnameField, fnameField, emailField, idField;
		JTextArea infoField;
		Box panel = new Box(BoxLayout.Y_AXIS);
		Element newStudent;

		Box box;

		if (student != null)
		{
			fname = DOMUtil.findChildValue( student, "first" );
			lname = DOMUtil.findChildValue( student, "last" );
			email = DOMUtil.findChildValue( student, "email" );
			info = DOMUtil.findChildValue( student, "info" );
			id = student.getAttribute("id");
		}

		fnameField = new JTextField( fname, 20 );
		lnameField = new JTextField( lname, 20 );
		emailField = new JTextField( email, 30 );
		idField = new JTextField( id, 15 );
		infoField = new JTextArea( info, 3, 30 );
		infoField.setLineWrap( true );
		infoField.setWrapStyleWord( true );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Last name"));
		box.add( Box.createHorizontalStrut(5));
		box.add( lnameField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("First name"));
		box.add( Box.createHorizontalStrut(5));
		box.add( fnameField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("ID"));
		box.add( Box.createHorizontalStrut(5));
		box.add( idField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("email"));
		box.add( Box.createHorizontalStrut(5));
		box.add( emailField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("info"));
		box.add( Box.createHorizontalStrut(5));
		box.add( infoField );
		panel.add( box );

		optPane = new JOptionPane( panel,
			JOptionPane.PLAIN_MESSAGE,
			JOptionPane.OK_CANCEL_OPTION );
		JDialog dialog =
			optPane.createDialog( list.getTopLevelAncestor(),
				"Student Information" );
		dialog.setVisible(true);

		Object val = optPane.getValue();

		boolean	changed = false;
		if (val != null)
		{
			if (val instanceof Integer)
			{
				int intVal = ((Integer)val).intValue();
				if (intVal != JOptionPane.CANCEL_OPTION &&
					intVal != JOptionPane.CLOSED_OPTION)
				{
					changed = true;
				}
			}
		}

		if (!changed) { return null; }

		String newLast = lnameField.getText();
		String newFirst = fnameField.getText();
		String newId = idField.getText();
		String newEmail = emailField.getText();
		String newInfo = infoField.getText();
		changed = !(newLast.equals(lname) &&
			newFirst.equals(fname) &&
			newId.equals(id) &&
			newEmail.equals(email) &&
			newInfo.equals(info));
		if (!changed) { return null; }

		student = updateStudent( student,
			newLast, newFirst, newId, newEmail, newInfo);

		return student;
	}

	public Element getResult( int studentNumber, int taskNumber )
	{
		Element task = (Element) (
			document.getElementsByTagName("task").item(taskNumber)
		);
		String id = task.getAttribute("id");
		Element student = (Element) (
			document.getElementsByTagName("student").
			item(studentNumber)
		);

		NodeList nl = student.getChildNodes();
		int i;

		for (i=0; i < nl.getLength(); i++)
		{
			if (nl.item(i).getNodeType() == Node.ELEMENT_NODE &&
				((Element)nl.item(i)).getAttribute("ref").equals(id))
			{
				return (Element) nl.item(i);
			}
		}
		return null;
	}

	public void setResult( Element result,
		String score, String comments )
	{
		Element newNode = document.createElement("result");
		comments = comments.replace('\n', '|');
		comments = DOMUtil.normalizeSpace( comments );

		Node	newText = document.createTextNode( comments );

		newNode.setAttribute("ref", result.getAttribute("ref"));
		newNode.setAttribute("score", score);
		newNode.appendChild( newText );

		result.getParentNode().replaceChild( newNode, result );
	}

	public void importTextFile()
	{
		JFileChooser chooser = new JFileChooser();
		int result;
		String fileName;
		String linebuf;
		String last, first, id, email;
		StringTokenizer tokenizer;
		BufferedReader infile;

		chooser.setCurrentDirectory(
			new File( System.getProperty("user.dir") ) );
		result = chooser.showOpenDialog( list.getTopLevelAncestor() );
		if (result != JFileChooser.APPROVE_OPTION ||
			chooser.getSelectedFile() == null)
		{
			return;
		}

		fileName = chooser.getSelectedFile().getAbsolutePath();
		try
		{
			infile = new BufferedReader( new FileReader( fileName ) );
			while ((linebuf = infile.readLine()) != null)
			{
				tokenizer = new StringTokenizer( linebuf, "\t" );
				last = first = id = email = "";
				if (tokenizer.countTokens() >= 3)
				{
					try {
						last = tokenizer.nextToken().trim();
						first = tokenizer.nextToken().trim();
						id = tokenizer.nextToken().trim();
						email = tokenizer.nextToken().trim();
					}
					catch (NoSuchElementException e)
					{
					}
				}
				Element newStudent = updateStudent( null,
					last, first, id, email, "");
				addStudent( newStudent );
			}
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
		}
	}

	protected boolean editStudent( int n )
	{
		Element student, newStudent;
		if (n >= 0)
		{
			student = (Element)
				document.getElementsByTagName("student").item(n);
		}
		else
		{
			student = null;
		}
		newStudent = studentDialog( student );
		if (newStudent != null)
		{
			if (student != null)
			{
				student.getParentNode().removeChild( student );
				model.removeElementAt( n );
			}
			n = addStudent( newStudent );
			list.setSelectedIndex( n );
			return true;
		}
		return false;
	}

	public int getSelectedIndex()
	{
		return list.getSelectedIndex();
	}
}
