import java.awt.*;
import javax.swing.*;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class TaskManager
{
	Document document;
	DoubleClickList	list;
	DefaultListModel model;

	public TaskManager( Document doc )
	{
		document = doc;
		model = new DefaultListModel();
		list = new DoubleClickList( model );
	}

	public Document getDocument()
	{
		return document;
	}

	public void setDocument( Document doc )
	{
		document = doc;
		clearList();
		populateList();
	}

	public DoubleClickList getList( )
	{
		return list;
	}

	public void clearList( )
	{
		if (model != null)
		{
			model.removeAllElements();
		}
	}

	public void populateList( )
	{
		NodeList taskNames;
		int i;
		if (model == null)
		{
			System.out.println("no model");
			return;
		}
		taskNames = document.getElementsByTagName( "task" );
		for (i=0; i < taskNames.getLength(); i++)
		{
			Element n = (Element) taskNames.item(i);
			model.addElement( DOMUtil.findValue( n ) +
				" [" +
				n.getAttribute("id")
				+ ": " +
				n.getAttribute("date") +
				"]" );
		}
	}

	public int addTask( int oldPos, Element oldTask, Element task )
	{
		NodeList	taskNames;
		Element		el;
		int			newPos;
		int			comparison;
		String		nodeDate;
		String		info, id, oldId, date;
		String		listStr;

		taskNames = document.getElementsByTagName( "task" );

		info = DOMUtil.findValue( task );

		id = task.getAttribute("id");
		if (oldPos != -1)
		{
			oldId = oldTask.getAttribute("id");
		}
		else
		{
			oldId = id;
		}

		date = task.getAttribute("date");

		listStr = info + " [" + id + ": " + date + "]";

		if (taskNames.getLength() != 0)
		{
			/* Find where the new element belongs */
			newPos = 0;
			while (newPos < taskNames.getLength())
			{
				nodeDate = ((Element) taskNames.item(newPos))
					.getAttribute("date");
				comparison = date.compareTo(nodeDate);
				if (comparison <= 0)
				{
					break;
				}
				newPos++;
			}
		}
		else
		{
			newPos = 0;
		}

		if (newPos != oldPos)
		{
			/* Task has moved. Insert it in new location,
				and update all students accordingly */
			Element taskListElement =
				(Element) document.getElementsByTagName("task-list").
				item(0);
			if (oldTask != null)
			{
				taskListElement.removeChild( oldTask );
			}
			taskListElement.insertBefore(
				task, taskNames.item(newPos));
			moveStudentResults( oldPos, newPos, id );
		}
		if (oldPos >= 0)
		{
			model.removeElementAt( oldPos );
		}
		model.insertElementAt( listStr, newPos );

		return newPos;
	}

	protected void moveStudentResults(int oldPos,
		int newPos, String id)
	{
		NodeList studentList =
			document.getElementsByTagName("student");
		int			i;
		Element		newResult;

		/* Create a new result (in case this is a new task) */

		for (i=0; i < studentList.getLength(); i++)
		{
			Node student = studentList.item(i);

			if (oldPos == -1)
			{
				newResult = document.createElement("result");
				newResult.setAttribute( "ref", id );
				newResult.setAttribute( "score", "");
				student.insertBefore( newResult,
					getResult( student, newPos ));
			}
			else
			{
				/* This is hard to understand, but the
					"newChild" node is the one that is currently
					at the old position, and it is to be inserted
					before the "refChild", which is currently
					at the new position */
				student.insertBefore(
					getResult( student, oldPos ),
					getResult( student, newPos )
				);
			}
		}
	}

	protected Node getResult( Node parentNode, int which )
	{
		int i;
		Node	result = null;
		Node	node = parentNode.getFirstChild();
		
		i = 0;
		while (node != null)
		{
			if (node.getNodeType() == Node.ELEMENT_NODE &&
				((Element)node).getTagName().equals("result"))
			{
				if (i == which)
				{
					break;
				}
				i++;
			}
			node = node.getNextSibling();
		}
		return node;
	}
	
	protected void changeStudentResultIds( String oldId, String id )
	{
		NodeList resultList = document.getElementsByTagName( "result" );
		int		i;
		for (i=0; i < resultList.getLength(); i++)
		{
			Element r = (Element) resultList.item(i);
			if (r.getAttribute("ref").equals(oldId))
			{
				r.setAttribute("ref", id);
			}
		}
	}

	public boolean deleteTask()
	{
		int		n = list.getSelectedIndex();
		int		result;
		String	id;
		Element	task;
		NodeList resultList;

		if (n < 0)
		{
			return false;
		}
		task = (Element)
				document.getElementsByTagName("task").item(n);

		if (task == null) { return false; }
		result = JOptionPane.showConfirmDialog(
			list.getTopLevelAncestor(),
			"Really delete "
			+  DOMUtil.findValue( task ) +  " "
			+ "[" + task.getAttribute("id") + "]?");
		if (result == JOptionPane.YES_OPTION)
		{
			id = task.getAttribute("id");
			resultList = document.getElementsByTagName("result");
			for (int i=0; i < resultList.getLength(); i++)
			{
				Element r = (Element) resultList.item(i);
				if (r.getAttribute("ref").equals(id))
				{
					r.getParentNode().removeChild(r);
				}
			}
			task.getParentNode().removeChild( task );
			model.removeElementAt( n );
			list.clearSelection();
			return true;
		}
		return false;
	}

	public Element updateTask( Element baseTask,
		String info,
		String id,
		String date,
		String type,
		String max,
		String weight,
		String recorded)
	{
		if (baseTask != null)
		{
			baseTask.setAttribute( "id", id );
			baseTask.setAttribute( "date", date );
			baseTask.setAttribute( "weight", weight );
			baseTask.setAttribute( "max", max );
			baseTask.setAttribute( "recorded", recorded );
			baseTask.setAttribute( "type", type );
			DOMUtil.setText( baseTask, "info", info );
		}
		else
		{
			baseTask = document.createElement( "task" );
			baseTask.setAttribute( "id", id );
			baseTask.setAttribute( "date", date );
			baseTask.setAttribute( "weight", weight );
			baseTask.setAttribute( "max", max );
			baseTask.setAttribute( "recorded", recorded );
			baseTask.setAttribute( "type", type );
			baseTask.appendChild( document.createTextNode( info ) );
		}

		return baseTask;
	}

	protected Element taskDialog( int position, Element task )
	{
		String	info="";
		String	weight="";
		String	max="";
		String	type="";
		String	id="";
		String	date="";
		String	recorded="";
		String	taskTypes[] = new String[] {
				"lab",
				"homework",
				"project",
				"attendance",
				"extra-credit",
				"test",
				"quiz",
				"midterm",
				"final"
		};

		JComboBox typeField = new JComboBox(
			taskTypes);
		JComboBox recordedField = new JComboBox(
			new String[] { "no", "yes" } );

		JOptionPane optPane;
		JTextField infoField, idField, dateField, weightField, maxField;

		Box panel = new Box(BoxLayout.Y_AXIS);

		Box box;

		typeField.setEditable( false );
		typeField.setSelectedIndex( 0 );
		recordedField.setEditable( false );
		recordedField.setSelectedIndex( 0 );
		if (task != null)
		{
			weight = task.getAttribute( "weight" );
			max = task.getAttribute( "max" );
			recorded = task.getAttribute( "recorded ");
			id = task.getAttribute("id");
			date = task.getAttribute("date");
			info = DOMUtil.findValue( task );
			type = task.getAttribute("type");
			recorded = task.getAttribute("recorded");
		}
		else
		{
			type="lab";
			recorded="no";
		}
		setComboByValue( typeField, type );
		setComboByValue( recordedField, recorded);

		infoField = new JTextField( info, 30 );
		idField = new JTextField( id, 10 );
		dateField = new JTextField( date, 20 );
		maxField = new JTextField( max, 20 );
		weightField = new JTextField( weight, 10 );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Task name"));
		box.add( Box.createHorizontalStrut(5));
		box.add( infoField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Date"));
		box.add( Box.createHorizontalStrut(5));
		box.add( dateField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("ID"));
		box.add( Box.createHorizontalStrut(5));
		box.add( idField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Type"));
		box.add( Box.createHorizontalStrut(5));
		box.add( typeField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Max. score"));
		box.add( Box.createHorizontalStrut(5));
		box.add( maxField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Weight"));
		box.add( Box.createHorizontalStrut(5));
		box.add( weightField );
		panel.add( box );

		box = new Box( BoxLayout.X_AXIS );
		box.add(new JLabel("Recorded"));
		box.add( Box.createHorizontalStrut(5));
		box.add( recordedField );
		panel.add( box );

		optPane = new JOptionPane( panel,
			JOptionPane.PLAIN_MESSAGE,
			JOptionPane.OK_CANCEL_OPTION );
		JDialog dialog =
			optPane.createDialog( list.getTopLevelAncestor(),
				"Task Information" );
		dialog.setVisible(true);

		Object val = optPane.getValue();

		boolean	changed = false;
		if (val != null)
		{
			if (val instanceof Integer)
			{
				int intVal = ((Integer)val).intValue();
				if (intVal != JOptionPane.CANCEL_OPTION &&
					intVal != JOptionPane.CLOSED_OPTION)
				{
					changed = true;
				}
			}
		}

		if (!changed) { return null; }

		String newInfo = infoField.getText();
		String newDate = dateField.getText();
		String newId = idField.getText();
		String newMax = maxField.getText();
		String newWeight = weightField.getText();
		String newType = (String) typeField.getSelectedItem();
		String newRecorded = (String) recordedField.getSelectedItem();
		changed = !(newInfo.equals(info) &&
			newDate.equals(date) &&
			newId.equals(id) &&
			newWeight.equals(weight) &&
			newMax.equals(max) &&
			newType.equals(type) &&
			newRecorded.equals(recorded));
		if (!changed) { return null; }

		if (isDuplicate( position, newId ))
		{
			return null;
		}

		if (!id.equals(newId))
		{
			changeStudentResultIds( id, newId );
		}

		task = updateTask( task,
			newInfo, newId, newDate, newType, newMax,
			newWeight, newRecorded);
		return task;
	}
	
	protected boolean isDuplicate( int position, String newId )
	{
		NodeList taskNames = document.getElementsByTagName("task");
		
		for (int i=0; i < taskNames.getLength(); i++)
		{
			String nodeId =
					((Element) taskNames.item(i)).getAttribute("id");
			if (nodeId.equals(newId) && i != position)
			{
				JOptionPane.showMessageDialog(
					list.getTopLevelAncestor(),
					"Duplicate ID with " +
						DOMUtil.findValue(
							(Element) taskNames.item(i)
						)
				);
				return true;
			}
		}
		return false;
	}


	public Element getResult( int studentNumber, int taskNumber )
	{
		Element task = (Element) (
			document.getElementsByTagName("task").item(taskNumber)
		);
		String id = task.getAttribute("id");
		Element student = (Element) (
			document.getElementsByTagName("student").
			item(studentNumber)
		);

		NodeList nl = student.getChildNodes();
		int i;

		for (i=0; i < nl.getLength(); i++)
		{
			if (nl.item(i).getNodeType() == Node.ELEMENT_NODE &&
				((Element)nl.item(i)).getAttribute("ref").equals(id))
			{
				return (Element) nl.item(i);
			}
		}
		return null;
	}

	protected void setComboByValue( JComboBox combo, String value )
	{
		int i;
		for (i=0; i < combo.getItemCount(); i++)
		{
			if (value.equals( (String)combo.getItemAt( i ) ))
			{
				combo.setSelectedIndex(i);
				break;
			}
		}
	}
			
	public void setResult( Element result,
		String score, String comments )
	{
		Element newNode = document.createElement("result");
		comments = comments.replace('\n', '|');
		Node	newText = document.createTextNode( comments );

		newNode.setAttribute("ref", result.getAttribute("ref"));
		newNode.setAttribute("score", score);
		newNode.appendChild( newText );

		result.getParentNode().replaceChild( newNode, result );
	}

	protected boolean editTask( int n )
	{
		Element task, newTask, oldTask;
		if (n >= 0)
		{
			task = (Element)
				document.getElementsByTagName("task").item(n);
		}
		else
		{
			task = null;
		}
		newTask = taskDialog( n, task );

		if (newTask != null)
		{
			oldTask = task;
			n = addTask( n, oldTask, newTask );
			list.setSelectedIndex( n );
			return true;
		}
		return false;
	}

	public int getSelectedIndex()
	{
		return list.getSelectedIndex();
	}
}
